@@ -7,16 +7,15 @@ import re
 import difflib
 from six import string_types
 
-from . import glyphs
-
+from .models import glyphs
 from .models import (
     BoxSelectionOverlay, BoxSelectTool, BoxZoomTool, CategoricalAxis,
     ColumnDataSource, TapTool, CrosshairTool, DataRange1d, DatetimeAxis,
     FactorRange, Grid, HoverTool, LassoSelectTool, Legend, LinearAxis,
     LogAxis, PanTool, Plot, PolySelectTool,
     PreviewSaveTool, Range, Range1d, ResetTool, ResizeTool, Tool,
-    WheelZoomTool,
-)
+    WheelZoomTool)
+
 from .properties import ColorSpec, Datetime
 from .utils import nice_join
 import warnings
@@ -116,7 +115,7 @@ def _match_data_params(argnames, glyphclass, datasource, serversource,
             # both strings and certain iterables are valid colors.
             glyph_val = val
         elif isinstance(val, string_types):
-            if glyphclass == glyphs.Text:
+            if glyphclass == glyphs.Text: # XXX: issubclass()
                 # TODO (bev) this is hacky, now that text is a DataSpec, it has to be a sequence
                 glyph_val = [val]
             elif serversource is None and val not in datasource.column_names:
@@ -319,6 +318,67 @@ def _tool_from_string(name):
 
         raise ValueError("unexpected tool name '%s', %s tools are %s" % (name, text, nice_join(matches)))
 
+
+def _process_tools_arg(plot, tools):
+    """ Adds tools to the plot object skipping those specified in
+    tools_to_remove.
+
+    Args:
+        plot (Plot): instance of a plot object
+        tools (seq[Tool or str]|str): list of tool types or string listing the
+            tool names. Those are converted using the _tool_from_string
+            function. I.e.: `wheel_zoom,box_zoom,reset`.
+
+    Returns:
+        list of Tools objects added to plot
+    """
+    tool_objs = []
+    temp_tool_str = ""
+    repeated_tools = []
+    removed_tools = []
+    tools_to_remove = []
+    # Remove pan/zoom tools in case of categorical axes
+    if isinstance(plot.x_range, FactorRange) or \
+            isinstance(plot.y_range, FactorRange):
+        tools_to_remove = ['pan', 'zoom']
+
+    if isinstance(tools, (list, tuple)):
+        for tool in tools:
+            if isinstance(tool, Tool):
+                tool_objs.append(tool)
+            elif isinstance(tool, string_types):
+                temp_tool_str += tool + ','
+            else:
+                raise ValueError("tool should be a string or an instance of Tool class")
+        tools = temp_tool_str
+
+    for tool in re.split(r"\s*,\s*", tools.strip()):
+        # re.split will return empty strings; ignore them.
+        if tool == "":
+            continue
+
+        if any(x in tool for x in tools_to_remove):
+            removed_tools.append(tool)
+            continue
+
+        tool_obj = _tool_from_string(tool)
+        tool_objs.append(tool_obj)
+
+    for typename, group in itertools.groupby(
+            sorted([tool.__class__.__name__ for tool in tool_objs])):
+        if len(list(group)) > 1:
+            repeated_tools.append(typename)
+
+    if repeated_tools:
+        warnings.warn("%s are being repeated" % ",".join(repeated_tools))
+
+    if removed_tools:
+        warnings.warn("categorical plots do not support pan and zoom operations.\n"
+                      "Removing tool(s): %s" % ', '.join(removed_tools))
+
+    return tool_objs
+
+
 def _new_xy_plot(x_range=None, y_range=None, plot_width=None, plot_height=None,
                  x_axis_type="auto", y_axis_type="auto",
                  x_axis_location="below", y_axis_location="left",
