@@ -7,8 +7,8 @@ from flask import request
 from bokeh import protocol
 
 from .bbauth import (
-    check_read_authentication_and_create_client,
-    check_write_authentication_and_create_client
+    check_read_authentication,
+    check_write_authentication
 )
 from ..app import bokeh_app
 from ..crossdomain import crossdomain
@@ -23,12 +23,12 @@ def init_bokeh(clientdoc):
 # Management Functions
 
 @bokeh_app.route("/bokeh/bb/<docid>/reset", methods=['GET'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def reset(docid):
     ''' Reset a specified :class:`Document <bokeh.document.Document>`.
 
     Deletes all stored objects except for the current
-    :class:`PlotContext <bokeh.objects.PlotContext>`, which has all of
+    :class:`PlotContext <bokeh.models.PlotContext>`, which has all of
     its children removed.
 
     :param docid: id of the :class:`Document <bokeh.document.Document>`
@@ -49,7 +49,7 @@ def reset(docid):
     return 'success'
 
 @bokeh_app.route("/bokeh/bb/<docid>/rungc", methods=['GET'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def rungc(docid):
     ''' Run the Bokeh Server garbage collector for a given
     :class:`Document <bokeh.document.Document>`.
@@ -66,7 +66,7 @@ def rungc(docid):
     return 'success'
 
 @bokeh_app.route("/bokeh/bb/<docid>/callbacks", methods=['POST'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def callbacks_post(docid):
     ''' Update callbacks for a given :class:`Document <bokeh.document.Document>`.
 
@@ -85,7 +85,7 @@ def callbacks_post(docid):
     return make_json(protocol.serialize_json(jsondata))
 
 @bokeh_app.route("/bokeh/bb/<docid>/callbacks", methods=['GET'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def callbacks_get(docid):
     ''' Retrieve callbacks for a given :class:`Document <bokeh.document.Document>`.
 
@@ -104,7 +104,7 @@ def callbacks_get(docid):
 
 # bulk upsert
 @bokeh_app.route("/bokeh/bb/<docid>/bulkupsert", methods=['POST'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def bulk_upsert(docid):
     ''' Update or insert new objects for a given :class:`Document <bokeh.document.Document>`.
 
@@ -149,7 +149,7 @@ def ws_delete(clientdoc, models):
 
 # backbone functionality
 @bokeh_app.route("/bokeh/bb/<docid>/<typename>/", methods=['POST'])
-@check_write_authentication_and_create_client
+@check_write_authentication
 def create(docid, typename):
     ''' Update or insert new objects for a given :class:`Document <bokeh.document.Document>`.
 
@@ -170,7 +170,7 @@ def create(docid, typename):
     ws_update(clientdoc, modeldata)
     return protocol.serialize_json(modeldata[0]['attributes'])
 
-@check_read_authentication_and_create_client
+@check_read_authentication
 def _bulkget(docid, typename=None):
     clientdoc = bokeh_app.backbone_storage.get_document(docid)
     prune(clientdoc)
@@ -224,7 +224,7 @@ def _handle_specific_model(docid, typename, id, method):
         return delete(docid, typename, id)
 
 # route for working with individual models
-@bokeh_app.route("/bokeh/bb/<docid>/<typename>/<id>/", methods=['GET'])
+@bokeh_app.route("/bokeh/bb/<docid>/<typename>/<id>/", methods=['GET', 'OPTIONS'])
 def _handle_specific_model_get(docid, typename, id):
     ''' Retrieve a specific model with a given id and typename for a
     given :class:`Document <bokeh.document.Document>`.
@@ -240,21 +240,6 @@ def _handle_specific_model_get(docid, typename, id):
     '''
     return _handle_specific_model(docid, typename, id, request.method)
 
-@bokeh_app.route("/bokeh/bb/<docid>/<typename>/<id>/", methods=['OPTIONS'])
-def _handle_specific_model_options(docid, typename, id):
-    ''' Retrieve crossdomain options for a specific model with a
-    given id and typename for a given :class:`Document <bokeh.document.Document>`.
-
-    :param docid: id of the :class:`Document <bokeh.document.Document>`
-        to update or insert into
-    :param typename: the type of objects to find and return
-    :param id: unique id of the object to retrieve
-
-    :status 200:
-
-    '''
-    return _handle_specific_model(docid, typename, id, request.method)
-
 @bokeh_app.route("/bokeh/bb/<docid>/<typename>/<id>/", methods=['PUT'])
 def _handle_specific_model_put(docid, typename, id):
     ''' Update a specific model with a given id and typename for a
@@ -305,14 +290,14 @@ def _handle_specific_model_delete(docid, typename, id):
 
 
 # individual model methods
-@check_read_authentication_and_create_client
+@check_read_authentication
 def getbyid(docid, typename, id):
     clientdoc = bokeh_app.backbone_storage.get_document(docid)
     prune(clientdoc)
     attr = clientdoc.dump(clientdoc._models[id])[0]['attributes']
     return make_json(protocol.serialize_json(attr))
 
-@check_write_authentication_and_create_client
+@check_write_authentication
 def update(docid, typename, id):
     """we need to distinguish between writing and patching models
     namely in writing, we shouldn't remove unspecified attrs
@@ -324,10 +309,13 @@ def update(docid, typename, id):
     init_bokeh(clientdoc)
     log.info("updating")
     modeldata = protocol.deserialize_json(request.data.decode('utf-8'))
+    ### horrible hack, we need to pop off the noop object if it exists
+    modeldata.pop('noop', None)
     # patch id is not passed...
     modeldata['id'] = id
     modeldata = {'type' : typename,
                  'attributes' : modeldata}
+
     clientdoc.load(modeldata, events='existing', dirty=True)
     log.info("done")
     log.info("saving")
@@ -340,7 +328,7 @@ def update(docid, typename, id):
     # all go out over the websocket channel
     return make_json(protocol.serialize_json({'noop' : True}))
 
-@check_write_authentication_and_create_client
+@check_write_authentication
 def delete(docid, typename, id):
     clientdoc = bokeh_app.backbone_storage.get_document(docid)
     prune(clientdoc)
@@ -348,21 +336,3 @@ def delete(docid, typename, id):
     clientdoc.del_obj(docid, model)
     ws_delete(clientdoc, [model])
     return make_json(protocol.serialize_json(clientdoc.dump(model)[0]['attributes']))
-
-
-# rpc route
-@bokeh_app.route("/bokeh/bb/rpc/<docid>/<typename>/<id>/<funcname>/", methods=['POST', 'OPTIONS'])
-@crossdomain(origin="*", methods=['POST'], headers=['BOKEH-API-KEY', 'Continuum-Clientid', 'Content-Type'])
-@check_write_authentication_and_create_client
-def rpc(docid, typename, id, funcname):
-    clientdoc = bokeh_app.backbone_storage.get_document(docid)
-    prune(clientdoc)
-    model = clientdoc._models[id]
-    data = protocol.deserialize_json(request.data.decode('utf-8'))
-    args = data.get('args', [])
-    kwargs = data.get('kwargs', {})
-    result = getattr(model, funcname)(*args, **kwargs)
-    log.debug("rpc, %s, %s", docid, typename)
-    changed = bokeh_app.backbone_storage.store_document(clientdoc)
-    ws_update(clientdoc, changed)
-    return make_json(protocol.serialize_json(result))
