@@ -3,9 +3,9 @@ from __future__ import print_function
 from bokeh.browserlib import view
 from bokeh.document import Document
 from bokeh.embed import file_html
-from bokeh.glyphs import Circle
-from bokeh.objects import (
-    Plot, DataRange1d, LinearAxis, ColumnDataSource, Glyph, PanTool, WheelZoomTool
+from bokeh.models.glyphs import Circle
+from bokeh.models import (
+    Plot, DataRange1d, LinearAxis, ColumnDataSource, PanTool, WheelZoomTool
 )
 from bokeh.resources import INLINE
 
